/*
Air rifle trigger program
28/04/14
*/

// Definitions
sbit trig_out at PORTC.B0;
unsigned short int tmr0_start;

// Interrupt handler
void interrupt()
{
     // Need to determine what interrupt occurred
     if(C1IF_bit==1)
     {
      // Comparator interrupt
      C1IE_bit=0; // disable comparator interrupt
      C1IF_bit=0; // clear flag
      TMR0=tmr0_start; // load timer with starting value
      T0IF_bit=0; // clear flag
      T0IE_bit=1; // enable timer interrupt
     } else
     {
      // Timer overflow interrupt
      T0IE_bit=0; // disable interrupt
      T0IF_bit=0; // clear flag
      PORTC.B0=1; // set output high
      delay_us(50); // give delay to ensure trigger
      PORTC.B0=0; // set output low
      C1IF_bit=0; // clear flag
      C1IE_bit=1; // enable comparator interrupt
     }
}

// Initialise ports and pins
void init()
{
     // Set up ports
     ANSEL=0b00100011; // RA0,1 & RC1 as inputs
     ANSELH=0;
     TRISA.B0=1; // C1IN+
     TRISA.B1=1; // C1IN0-
     TRISA.B2=0; // C1OUT (monitor)
     TRISC.B0=0; // Trigger output
     PORTC.B0=0;
     TRISC.B1=1; // potentiometer input
     
     // Set up ADC
     ADCON0=0b00010101;
     ADCON1=0b00100000;
     
     // Set up Timer 0
     OPTION_REG=0b10000011;

     // Set up comparator 1
     CM1CON0=0b10100000;
     delay_us(100); // delay to give inputs time to settle
     
     // Set up interrupts
     T0IF_bit=0; // clear flags
     C1IF_bit=0; // clear flags
     T0IE_bit=0; // don't enable timer yet
     C1IE_bit=1; // enable comparator interrupt
     PEIE_bit=1; // enable ext. interrupts
     GIE_bit=1; // enable global interrupts
}

void main()
{
     // Initalise stuff
     init();
     
     // Endless loop to read the potentiometer input and
     // update the initial value to use for the timer
     while(1)
     {
      ADCON0.B1=1; // start conversion
      while(ADCON0.B1==1){} // wait until conversion completes
      tmr0_start=255-ADRESH; // assign result to tmr0_start
      delay_ms(100); // delay before next loop
     }
     
}